<?xml version="1.0" encoding="ISO-8859-1"?>

<xsl:stylesheet version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

    <xsl:output method="xml"/>
    
    <xsl:template match="/">
        <xsl:text disable-output-escaping="yes">
          <![CDATA[
            <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
            "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
          ]]>
        </xsl:text>
        <html xmlns="http://www.w3.org/1999/xhtml">
            <head>
                <meta http-equiv="Content-Type" content="text/html;charset=utf-8" />
                <title>Manual</title>
                <link rel="stylesheet" type="text/css" href="manual.css" />
            </head>
            <body>
                <table class="main">
                    <tr>
                        <td class="toc">
                            <h3>Index</h3>
                            <xsl:call-template name="toc"/>
                        </td>
                        <td class="manual">
                            <h1>Manual</h1>
                            <xsl:apply-templates select="*/data-type|*/token|*/mcomp"/>
                        </td>
                    </tr>
                </table>
            </body>
        </html>
    </xsl:template>
    
    <!--
        Table of Contents
    -->
    
    <xsl:template name="toc">
        <ul>
            <xsl:for-each select="*/data-type|*/token|*/mcomp">
                <xsl:sort select="concat(symbol,name)"/>
                <xsl:choose>
                    <xsl:when test="name(.) = 'data-type'">
                        <xsl:call-template name="tocItem_DataType"/>
                    </xsl:when>
                    <xsl:when test="name(.) = 'mcomp'">
                        <xsl:call-template name="tocItem_MComp"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="tocItem_Token"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:for-each>
        </ul>
    </xsl:template>
    
    <xsl:template name="tocItem_DataType">
        <li class="tocItem_DataType">
            <a>
                <xsl:attribute name="href">
                    <xsl:text>#</xsl:text><xsl:value-of select="@id"/>
                </xsl:attribute>
                <xsl:value-of select="name"/>
            </a>
        </li>
    </xsl:template>
    
    <xsl:template name="tocItem_Token">
        <li class="tocItem_Token">
            <a>
                <xsl:attribute name="href">
                    <xsl:text>#</xsl:text><xsl:value-of select="@id"/>
                </xsl:attribute>
                <xsl:value-of select="symbol"/>
            </a>
        </li>
    </xsl:template>
    
    <xsl:template name="tocItem_MComp">
        <li class="tocItem_MComp">
            <a>
                <xsl:attribute name="href">
                    <xsl:text>#</xsl:text><xsl:value-of select="@id"/>
                </xsl:attribute>
                <xsl:value-of select="name"/>
            </a>
        </li>
        <xsl:for-each select="token">
            <xsl:sort select="concat(symbol,name)"/>
            <xsl:call-template name="tocItem_MComp_Token"/>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template name="tocItem_MComp_Token">
        <li class="tocItem_MComp_Token">
            <a>
                <xsl:attribute name="href">
                    <xsl:text>#</xsl:text><xsl:value-of select="@id"/>
                </xsl:attribute>
                <xsl:value-of select="../namespace"/>:<xsl:value-of select="symbol"/>
            </a>
        </li>
    </xsl:template>
    
    <!--
        Manual
    -->
    
    <!--bookmark -->
    <xsl:template name="bookmark">
        <a>
            <xsl:attribute name="id">
                <xsl:value-of select="@id"/>
            </xsl:attribute>
            <xsl:attribute name="name">
                <xsl:value-of select="@id"/>
            </xsl:attribute>
            <xsl:text> </xsl:text>
        </a>
    </xsl:template>
    
    <!-- title -->
    <xsl:template match="title">
        <h3 class="title"><xsl:value-of select="."/></h3>
    </xsl:template>
    
    <!-- code -->
    <xsl:template match="code">
        <p class="code"><xsl:value-of select="."/></p>
    </xsl:template>
    
    <!-- remark -->
    <xsl:template match="remark">
        <p class="remark"><xsl:apply-templates select="link|text()"/></p>
    </xsl:template>
    
    <!-- link -->
    <xsl:template name="link" match="link">
        <a>
            <xsl:attribute name="href">
                <xsl:text>#</xsl:text><xsl:value-of select="@ref"/>
            </xsl:attribute>
            <xsl:choose>
                <xsl:when test=". = ''">
                    <xsl:variable name="ref" select="@ref"/>
                    <xsl:value-of select="//*[@id=$ref]/name|//*[@id=$ref]/symbol"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="."/>
                </xsl:otherwise>
            </xsl:choose>
        </a>
    </xsl:template>
    
    <!-- in -->
    <xsl:template match="in">
        <b>In: </b>
        <xsl:for-each select="link">
            <xsl:call-template name="link"/>
            <xsl:if test="position() &lt; count(../link)">, </xsl:if>
        </xsl:for-each>
    </xsl:template>
    
    <!-- out -->
    <xsl:template match="out">
        <b> Out: </b>
        <xsl:for-each select="link">
            <xsl:call-template name="link"/>
        </xsl:for-each>
    </xsl:template>
    
    <!-- use-case/remark -->
    <xsl:template match="use-case/remark">
        <br /><span class="useCaseRemark"><xsl:apply-templates select="link|text()"/></span>
    </xsl:template>
    
    <!-- use-case -->
    <xsl:template match="use-case">
        <li>
            <xsl:apply-templates select="in|out|remark"/>
        </li>
    </xsl:template>
    
    <!-- usage -->
    <xsl:template match="usage">
        <ul class="usage">
            <xsl:apply-templates select="use-case"/>
        </ul>
    </xsl:template>
    
    <!-- section -->
    <xsl:template match="section">
        <xsl:apply-templates select="title|remark|code|usage"/>
    </xsl:template>
    
    <!-- data-type -->
    <xsl:template match="data-type">
        <xsl:call-template name="bookmark"/>
        <div class="type">Data type</div>
        <div class="dataType">
            <h2 class="name"><xsl:value-of select="name"/></h2>
            <xsl:apply-templates select="section"/>
        </div>
    </xsl:template>
    
    <!-- token -->
    <xsl:template match="token">
        <xsl:call-template name="bookmark"/>
        <div class="type">
            <xsl:choose>
                <xsl:when test="type = 'BOPERATOR'">
                    Binary operator
                </xsl:when>
                <xsl:when test="type = 'LUOPERATOR'">
                    Prefix unary operator
                </xsl:when>
                <xsl:when test="type = 'RUOPERATOR'">
                    Postfix unary operator
                </xsl:when>
                <xsl:when test="type = 'OPERAND'">
                    Value
                </xsl:when>
                <xsl:when test="type = 'FUNCTION'">
                    Function
                </xsl:when>
                <xsl:when test="type = 'LPAREN'">
                    Left parenthesis
                </xsl:when>
                <xsl:when test="type = 'RPAREN'">
                    Right parenthesis
                </xsl:when>
                <xsl:when test="type = 'NOOP'">
                    No-op
                </xsl:when>
                <xsl:otherwise>
                    Undefined
                </xsl:otherwise>
            </xsl:choose>
        </div>
        <div class="token">
            <h2 class="name">
                <xsl:value-of select="symbol"/>
                <xsl:if test="count(name) &gt; 0">
                    (<xsl:value-of select="name"/>)
                </xsl:if>
            </h2>
            <xsl:apply-templates select="section"/>
        </div>
    </xsl:template>
    
    <!-- mcomp -->
    <xsl:template match="mcomp">
        <xsl:call-template name="bookmark"/>
        <div class="type">MComp</div>
        <div class="mcomp">
            <h2 class="name"><xsl:value-of select="name"/></h2>
            <h3 class="title">Namespace</h3>
            <ul class="namespace">
                <li><xsl:value-of select="namespace"/></li>
            </ul>
            <xsl:if test="count(url) &gt; 0">
                <h3 class="title">URL</h3>
                <ul class="url">
                    <xsl:for-each select="url">
                        <li><xsl:value-of select="."/></li>
                    </xsl:for-each>
                </ul>
            </xsl:if>
            <xsl:apply-templates select="section"/>
            <xsl:apply-templates select="token"/>
        </div>
    </xsl:template>
    
</xsl:stylesheet>
